#!/bin/bash -e
#
# simple-sync-phone:bin/find-open-port-on-network.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/find-port-on-network-api.sh" || exit $?

declare -ri DEFAULT_SLEEP_DELAY=5

function usage {
  cat <<EOF
Find given port on local network.

Usage:

  $( basename "$0" ) --help
  $( basename "$0" ) --port PORT--pipe PIPE [--sleep-delay DELAY] [--debug]

Options
  -h,           --help      Display this messsage and exit.
                --debug     Show extra informations
  -P PORT,      --port PORT Port to use (ie: 2222)
                --pipe PIPE
  -S DELAY,     --sleep-delay DELAY
EOF
  read -n 1 -s -r -p 'Press enter to continue...'
}

function main {
  local param=
  local param_pipe=
  local param_port=
  local param_sleep_delay="${DEFAULT_SLEEP_DELAY}"

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-P'|'--port')
        param_port="$( P_param "$@" )" && shift || return $?
        if ! is_positive_integer "${param_port}" ; then
         echofatal '--port must be a positive number.'
         return 100
        fi
        if [ "${param_port}" -lt 1 ] || [ "${param_port}" -gt 65534 ] ; then
         echofatal '--port must be a number from 1 to 65534'
         return 100
        fi
        ;;

      '--pipe')
        param_pipe="$( P_param "$@" )" && shift || return $?
        ;;

      '-S'|'--sleep-delay')
        param_sleep_delay="$( P_param "$@" )" && shift || return $?
        if ! is_positive_integer "${param_port}" ; then
         echofatal '--port must be a positive number.'
         return 100
        fi
        ;;

      '--debug')
        declare -gx MESSAGE_API_SHOW_DEBUG='true'
        declare -gx VERBOSE='true'
        declare -gx __DEBUG_NMAP__='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        return 1
        ;;
    esac
  done

  if [ -z "${param_port}" ] ; then
    echofatal 'Missing --port option'
    usage >&2
    return 1
  fi
  if [ -z "${param_pipe}" ] ; then
    echofatal 'Missing --pipe option'
    usage >&2
    return 1
  fi

  local -i rc=0
  find_port_on_ipv4_network "${param_port}" >"${param_pipe}" || rc=$?

  sleep "${param_sleep_delay}"

  return "${rc}"
}

main "$@" || exit $?
